from illallangi.rdf.models.aircraft import Aircraft, AircraftKey
from illallangi.rdf.models.airline import Airline, AirlineKey
from illallangi.rdf.models.airport import Airport, AirportKey
from illallangi.rdf.models.alliance import Alliance, AllianceKey
from illallangi.rdf.models.course import Course, CourseKey
from illallangi.rdf.models.manufacturer import (
    Manufacturer,
    ManufacturerKey,
)
from illallangi.rdf.models.residence import Residence, ResidenceKey

__all__ = [
    "Aircraft",
    "AircraftKey",
    "Airline",
    "AirlineKey",
    "Airport",
    "AirportKey",
    "Alliance",
    "AllianceKey",
    "Course",
    "CourseKey",
    "Manufacturer",
    "ManufacturerKey",
    "Residence",
    "ResidenceKey",
]
